# pylint: disable=no-self-use
import os
import pytest

from miniscule import load_config


class TestConfigLoader:
    @pytest.mark.parametrize(
        'res,stream', [(None, '!or [null]'), (1, '!or [1]'),
                       ('hello', '!or [hello]'),
                       ('hello', '!or [null, hello]'),
                       ('hello', '!or [null, hello, null]'),
                       ('hello', '!or [null, hello, 1, null]')])
    def test_or_tag_selects_first_value_that_is_not_null(self, res, stream):
        assert load_config(stream) == res

    @pytest.mark.parametrize(
        'res,value', [(5000, '5000'), (500.1, '500.1'),
                      ('localhost', 'localhost')])
    def test_env_converts_environment_value_per_yaml_rules(self, res, value):
        os.environ['PORT'] = value
        assert load_config('!env PORT') == res
        del os.environ['PORT']

    def test_env_returns_none_if_argument_not_in_environment(self):
        assert load_config('!env HOST') is None



import os
import yaml


class ConfigLoader(yaml.SafeLoader):
    # pylint: disable=too-many-ancestors
    def __init__(self, stream):
        super(ConfigLoader, self).__init__(stream)
        yaml.add_constructor('!or', or_constructor, Loader=ConfigLoader)
        yaml.add_constructor('!env', env_constructor, Loader=ConfigLoader)


def or_constructor(loader, node):
    for expr in loader.construct_sequence(node):
        if expr is not None:
            return expr
    return None


def env_constructor(loader, node):
    name = loader.construct_yaml_str(node)
    if name in os.environ:
        return yaml.load(os.getenv(name))
    return None


def load_config(stream, Loader=ConfigLoader):
    return yaml.load(stream, Loader)


def read_config(path, Loader=ConfigLoader):
    with open(path, 'r') as stream:
        return load_config(stream, Loader=Loader)



from os.path import join, dirname
import re

import setuptools


def read_requirements(path):
    requirements = []
    with open(join(dirname(__file__), path), 'r') as handle:
        for line in handle:
            # grab what's before the comment
            line_content = line.split("#")[0].strip()
            if line_content:
                requirements.append(line_content)
    return requirements


def read_file(path):
    with open(path, 'r') as handle:
        return handle.read()

def read_version():
    s = read_file('VERSION')
    m = re.match(r'v(\d+\.\d+\.\d+)', s)
    return m.group(1)

long_description = read_file('README.md')
install_requires = read_requirements('requirements/install.txt')
test_requires = read_requirements('requirements/test.txt')
version = read_version()


setuptools.setup(
    name='miniscule',
    description='A YAML based, pluggable configuration library inspired by Aeson',
    long_description=long_description,
    version=version,
    url='https://gitlab.com/bartfrenk/miniscule/',
    author='Bart Frenk',
    author_email='bart.frenk@gmail.com',
    package_dir={'miniscule': 'src/miniscule'},
    classifiers=[
        'Programming Language :: Python :: 3',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python :: 3'
    ],
    install_requires=install_requires,
    test_requires=test_requires,
    packages=setuptools.find_packages('src'))
